#ifndef __ANSI_H
#define __ANSI_H

#include <stdlib.h>
#include <stdarg.h>
#include <stdio.h>
#include <sys/ioctl.h>

class CAnsi {
private:
	size_t mx, my;

public:
	CAnsi(): mx(0), my(0) { Resize(); }

	inline void Resize(void) {
		struct winsize w;

		if (ioctl(1, TIOCGWINSZ, &w)) {
			mx = 80;
			my = 25;
		} else {
			mx = w.ws_col;
			my = w.ws_row;
		}
	}

	enum attr_t {
		NORMAL = 0,
		BOLD = 1,
		BLACK = 30,
		RED,
		GREEN,
		YELLOW,
		BLUE,
		MAGENTA,
		CYAN,
		WHITE,		
	};

	inline void Print(const char *fmt, ...) {
		va_list ap;

		va_start(ap, fmt);
		vfprintf(stdout, fmt, ap);
		va_end(ap);
	}

	inline void Print(char ch) { fputc(ch, stdout); }
	inline void Cls(void) { Print("\e[0m\e[1;1H\e[2J"); }
	inline void ClsLine(void) { Print("\e[K"); }
	inline void Goto(int x, int y) { Print("\e[%d;%dH", y + 1, x + 1); }
	inline void Attr(int attr) { Print("\e[%dm", attr); }
	inline size_t SizeX(void) { return mx; }
	inline size_t SizeY(void) { return my; }
};

#endif
