#include <string>
#include <unistd.h>
#include <string.h>
#include <stdarg.h>
#include <stdio.h>
#include <errno.h>
#include <time.h>
#include "debug.h"
#include "wifi.h"
#include "ansi.h"
#include "log.h"

extern int errno;
extern CAnsi ansi;

CLog::CLog(): m_FP(NULL), m_Cnt(0)
{
	if (!(m_FP = fopen(LOGFILE, "a")))
		die("Can't open logfile: %s.", strerror(errno));

	Print("Log opened.");
}

CLog::~CLog()
{
	if (!m_FP)
		return;

	Print("Log closed.");
	fclose(m_FP);
}

void CLog::Print(const char *fmt, ...)
{
	va_list ap;
	char dstr[64];
	time_t t;
	struct tm *tm;

	if (!m_FP)
		return;

	t = time(NULL);
	tm = localtime(&t);
	strftime(dstr, sizeof(dstr), "[%Y-%m-%d %H:%M:%S] ", tm);

	fprintf(m_FP, "%s", dstr);

	va_start(ap, fmt);
	vfprintf(m_FP, fmt, ap);
	va_end(ap);

	fprintf(m_FP, "\n");

	if (++m_Cnt > 10) {
		m_Cnt = 0;
		fflush(m_FP);
		fdatasync(fileno(m_FP));
	}
}

void CLog::Log(CWifi::scan_t &scan)
{
	size_t i, num;

	if (!(num = scan.size()))
		return;

	for (i = 0; i < num; i++) {
		CWifi::ap_t ap = scan[i];

		Print("%d (%s), wep=%s, ess=%s, bssid=%s, chan=%2d, rate=%2d, signal=%3d, noise=%3d, snr=%3d", 
		    i + 1, ap.ssid.c_str(), ap.wep ? "yes" : "no", ap.ess ? "yes" : "no", ap.bssid, 
		    ap.chan, ap.rate, ap.signal, ap.noise, ap.signal - ap.noise);
	}
}
