#include <vector>
#include <signal.h>
#include <string.h>
#include <stdio.h>
#include <errno.h>
#include <time.h>
#include "debug.h"
#include "wifi.h"
#include "ansi.h"
#include "log.h"

extern int errno;
volatile int flag_term = 0;
using namespace std;

CAnsi ansi;
CLog log;

typedef struct {
	int active;
	time_t first, last;
	struct CWifi::ap_t ap;
} ap_t;

static void sighnd(int signo)
{
	if (signo == SIGWINCH) {
		ansi.Cls();
		die("Sorry, terminal size change is not supported.");
	}

	die("Terminating at signal %d.", signo);
	flag_term = signo;

	signal(signo, sighnd);
}

static void DrawTable(void)
{
	size_t i;

	ansi.Cls();
	ansi.Goto(0, 0);

	ansi.Goto(0, 0);
	ansi.Attr(CAnsi::BOLD); ansi.Attr(CAnsi::YELLOW);
	ansi.Print(" No");
	ansi.Attr(CAnsi::WHITE); ansi.Print(" | "); ansi.Attr(CAnsi::YELLOW);
	ansi.Print("FrstSeen");
	ansi.Attr(CAnsi::WHITE); ansi.Print(" | "); ansi.Attr(CAnsi::YELLOW);
	ansi.Print("LastSeen");
	ansi.Attr(CAnsi::WHITE); ansi.Print(" | "); ansi.Attr(CAnsi::YELLOW);
	ansi.Print("    BSS     ");
	ansi.Attr(CAnsi::WHITE); ansi.Print(" | "); ansi.Attr(CAnsi::YELLOW);
	ansi.Print("Chn");
	ansi.Attr(CAnsi::WHITE); ansi.Print(" | "); ansi.Attr(CAnsi::YELLOW);
	ansi.Print("Sig");
	ansi.Attr(CAnsi::WHITE); ansi.Print(" | "); ansi.Attr(CAnsi::YELLOW);
	ansi.Print("SNR");
	ansi.Attr(CAnsi::WHITE); ansi.Print(" | "); ansi.Attr(CAnsi::YELLOW);
	ansi.Print("SSID");
	ansi.Attr(CAnsi::WHITE);

	ansi.Goto(0, 1);
	ansi.Print("----+----------+----------+--------------+-----+-----+-----+");
	for (i = 60; i < ansi.SizeX(); i++)
		ansi.Print('-');
	ansi.Print('\n');
}

ap_t *Find(vector <ap_t> &aps, const CWifi::ap_t &wap)
{
	size_t i, sz;

	sz = aps.size();
	for (i = 0; i < sz; i++)
		if (!memcmp(aps[i].ap.bssid, wap.bssid, sizeof(wap.bssid)))
			return &(aps[i]);

	return NULL;
}

void UpdateAPs(vector <ap_t> &aps, const CWifi::scan_t &scan)
{
	size_t i, sz;

	sz = aps.size();
	for (i = 0; i < sz; i++) {
		aps[i].active = 0;
		aps[i].ap.signal = aps[i].ap.noise = -1;
	}

	sz = scan.size();
	for (i = 0; i < sz; i++) {
		ap_t *ap;

		if (!(ap = Find(aps, scan[i]))) {
			ap_t ap2;

			ap2.first = time(NULL);
			memcpy(ap2.ap.bssid, scan[i].bssid, sizeof(ap2.ap.bssid));
			aps.push_back(ap2);

			if (!(ap = Find(aps, scan[i])))
				die("Internal error: Can't find newly added AP.");
		}

		ap->active = 1;
		ap->last = time(NULL);
		ap->ap.ssid = scan[i].ssid;
		ap->ap.ess = scan[i].ess;
		ap->ap.wep = scan[i].wep;
		ap->ap.chan = scan[i].chan;
		ap->ap.rate = scan[i].rate;
		ap->ap.signal = scan[i].signal;
		ap->ap.noise = scan[i].noise;
	}
}

char *GetTime(time_t t)
{
	static char dstr[16];
	struct tm *tm;

	tm = localtime(&t);
	strftime(dstr, sizeof(dstr), "%H:%M:%S", tm);

	return dstr;
}

void SetAttr(ap_t &ap)
{
	if (ap.active) {
		ansi.Attr(CAnsi::BOLD);
		ansi.Attr(ap.ap.wep ? CAnsi::RED : CAnsi::GREEN);
	} else
		ansi.Attr(CAnsi::NORMAL);
}

void PrintAP(int i, ap_t &ap)
{
	SetAttr(ap);
	ansi.Print(" %2d", i + 1);
	ansi.Attr(CAnsi::BOLD); ansi.Attr(CAnsi::WHITE); ansi.Print(" | "); SetAttr(ap);
	ansi.Print(GetTime(ap.first));
	ansi.Attr(CAnsi::BOLD); ansi.Attr(CAnsi::WHITE); ansi.Print(" | "); SetAttr(ap);
	ansi.Print(GetTime(ap.last));
	ansi.Attr(CAnsi::BOLD); ansi.Attr(CAnsi::WHITE); ansi.Print(" | "); SetAttr(ap);
	ansi.Print("%s", ap.ap.bssid);
	ansi.Attr(CAnsi::BOLD); ansi.Attr(CAnsi::WHITE); ansi.Print(" | "); SetAttr(ap);
	ansi.Print("%c%2d", ap.ap.wep ? 'W' : ' ', ap.ap.chan);
	ansi.Attr(CAnsi::BOLD); ansi.Attr(CAnsi::WHITE); ansi.Print(" | "); SetAttr(ap);
	ansi.Print("%3d", ap.ap.signal);
	ansi.Attr(CAnsi::BOLD); ansi.Attr(CAnsi::WHITE); ansi.Print(" | "); SetAttr(ap);
	ansi.Print("%3d", ap.ap.signal - ap.ap.noise);
	ansi.Attr(CAnsi::BOLD); ansi.Attr(CAnsi::WHITE); ansi.Print(" | "); SetAttr(ap);
	ansi.Print("%s", ap.ap.ssid.c_str());
	ansi.Attr(CAnsi::NORMAL);
	ansi.Print('\n');
}

int main(int ac, char * const av[])
{
#if 0
	int oldnum = 0;
#endif
	CWifi wifi;
	vector <ap_t> aps;

	aps.clear();

	if (ansi.SizeX() < 80 || ansi.SizeY() < 3)
		die("Sorry, minimum terminal size is 80x3.");

	wifi.Init((ac > 1) ? av[1] : "wi0");

	signal(SIGWINCH, sighnd);
	signal(SIGTERM, sighnd);
	signal(SIGINT, sighnd);
	signal(SIGHUP, sighnd);

	DrawTable();

	while (!flag_term) {
		int i, start; //, num;
		CWifi::scan_t scan;

		wifi.Scan(scan);
		log.Log(scan);

		UpdateAPs(aps, scan);

		start = (aps.size() < (ansi.SizeY() - 3)) ? 0 : (aps.size() - (ansi.SizeY() - 3));
//		num = (aps.size() > ansi.SizeY() - 2) ? ansi.SizeY() - 2 : aps.size();
		for (i = start; i < (int) aps.size(); i++) {
			ansi.Goto(0, i + 2 - start);
			PrintAP(i, aps[i]);
		}

#if 0
		if (oldnum > num) {
			for (i = num; i < oldnum; i++) {
				ansi.Goto(0, i + 2);
				ansi.ClsLine();
			}
		}

		oldnum = num;
#endif
		usleep(500000);
	}

	wifi.Cleanup();
	ansi.Cls();

	return 0;
}
