#include <string>
#include <vector>
#include <string.h>
#include <errno.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/cdefs.h>
#include <sys/param.h>
#include <sys/ioctl.h>
#include <net/if.h>
#include <dev/ic/wi_ieee.h>
#include <net80211/ieee80211.h>
#include "debug.h"
#include "ansi.h"
#include "wifi.h"

using namespace std;

extern int errno;
extern CAnsi ansi;

int CWifi::Open(void)
{
	int fd;

	fd = socket(AF_INET, SOCK_DGRAM, 0);
	if (fd == -1)
		die("socket(): %s.", strerror(errno));

	return fd;
}

void CWifi::Close(int fd)
{
	close(fd);
}

int CWifi::GetFlags(void)
{
	struct ifreq ifr;
	int fd;

	memset(&ifr, 0, sizeof(ifr));
	strncpy(ifr.ifr_name, m_Name.c_str(), sizeof(ifr.ifr_name));

	fd = Open();
        if (ioctl(fd, SIOCGIFFLAGS, (caddr_t) &ifr) == -1)
		die("ioctl(SIOCGIFFLAGS): %s.", strerror(errno));
	Close(fd);

	return ifr.ifr_flags;
}

void CWifi::SetFlags(int flags)
{
	struct ifreq ifr;
	int fd;

	memset(&ifr, 0, sizeof(ifr));
	strncpy(ifr.ifr_name, m_Name.c_str(), sizeof(ifr.ifr_name));
	ifr.ifr_flags = flags;

	fd = Open();
        if (ioctl(fd, SIOCSIFFLAGS, (caddr_t) &ifr) == -1)
		die("ioctl(SIOCGIFFLAGS): %s.", strerror(errno));
	Close(fd);
}

void CWifi::SetVal(struct wi_req *wrq)
{
	struct ifreq ifr;
	int fd;

	memset(&ifr, 0, sizeof(ifr));
	strncpy(ifr.ifr_name, m_Name.c_str(), sizeof(ifr.ifr_name));
	ifr.ifr_data = (caddr_t) wrq;

	fd = Open();
	if (ioctl(fd, SIOCSWAVELAN, &ifr) == -1)
		die("ioctl(SIOCSWAVELAN): %s.", strerror(errno));
	Close(fd);
}

void CWifi::Init(string dev)
{
	m_Name = dev;
	m_Flags = GetFlags();
	if (!(m_Flags & IFF_UP))
		SetFlags(m_Flags | IFF_UP);
}

void CWifi::Cleanup(void)
{
	SetFlags(m_Flags & ~IFF_UP);
	m_Flags = -1;
	m_Name = string("");
}

void CWifi::Scan(scan_t &info)
{
	struct wi_req req;
	struct ifreq ifr;
	struct wi_apinfo *w;
	int i, naps;
	int fd;

	memset(&req, 0, sizeof(req));
	req.wi_type = WI_RID_SCAN_APS;
	req.wi_len = 4;
	req.wi_val[0] = 0x3FFF;		/* 14 channels */
	req.wi_val[1] = 0x0F;		/* rate */
	SetVal(&req);

	memset(&req, 0, sizeof(req));
	memset(&ifr, 0, sizeof(ifr));
	req.wi_type = WI_RID_READ_APS;
	req.wi_len = WI_MAX_DATALEN;
	strncpy(ifr.ifr_name, m_Name.c_str(), sizeof(ifr.ifr_name));
	ifr.ifr_data = (caddr_t) &req;

	fd = Open();
	for (i = 0; i < 10; i++) {
		int rs = ioctl(fd, SIOCGWAVELAN, &ifr);

		if (rs == -1 && errno != EINPROGRESS)
			die("ioctl(SIOCGWAVELAN): %s.", strerror(errno));
		else if (!rs)
			break;

		usleep(200000);
	}
	Close(fd);

	info.clear();

	naps = *(int *) req.wi_val;
	w = (struct wi_apinfo *) (((char *) &req.wi_val) + sizeof(int));
	for (i = 0; i < naps; i++, w++) {
		struct ap_t ap;
		int i;

		ap.signal = w->signal;
		ap.noise = w->noise;
		ap.chan = w->channel;
		ap.ssid = string(w->name, w->namelen);
		ap.ess = w->capinfo & IEEE80211_CAPINFO_ESS;
		ap.wep = w->capinfo & IEEE80211_CAPINFO_PRIVACY;

		if (w->rate == 0x0A)
			ap.rate = 1;
		else if (w->rate == 0x14)
			ap.rate = 2;
		else if (w->rate == 0x37)
			ap.rate = 5;
		else if (w->rate == 0x6E)
			ap.rate = 11;
		else
			ap.rate = 0;

		for (i = 0; i < 6; i++)
			sprintf(ap.bssid + i * 2, "%02X", (unsigned char) w->bssid[i]);

		info.push_back(ap);
	}
}
